@@ -344,11 +344,6 @@ def getCallsCode():
         CodeTemplates.template_helper_impl_decl % {}
     )
 
-    # TODO: If no variables used, move that code to helpers.
-    result.append(
-        CodeTemplates.template_call_cpython_function_fast_impl % {}
-    )
-
     for quick_call_used in sorted(quick_calls_used):
         args_decl = [
             "PyObject *arg%d" % d
