@@ -26,17 +26,16 @@ This is about collecting these constraints and to manage them.
 # Python3 compatibility.
 from logging import debug
 
-from nuitka import Options, Tracing, Utils
+from nuitka import Tracing, VariableRegistry
 from nuitka.__past__ import iterItems
-from nuitka.nodes.AssignNodes import StatementDelVariable
-from nuitka.VariableRegistry import isSharedLogically
+from nuitka.utils import Utils
 
 from .VariableTraces import (
-    VariableAssignTrace,
-    VariableInitTrace,
-    VariableMergeTrace,
-    VariableUninitTrace,
-    VariableUnknownTrace
+    VariableTraceAssign,
+    VariableTraceInit,
+    VariableTraceMerge,
+    VariableTraceUninit,
+    VariableTraceUnknown
 )
 
 signalChange = None
@@ -75,6 +74,7 @@ class VariableUsageTrackingMixin:
     def _makeVariableTraceOptimization(self, owner, variable_trace):
         variable = variable_trace.getVariable()
 
+        # TODO: Remove unused variables, does not work, why?
         if variable.isTempVariable():
             if variable.getOwner() is owner:
 
@@ -86,14 +86,6 @@ class VariableUsageTrackingMixin:
                         # owner.removeTempVariable( variable )
                         pass
 
-                # TODO: Something wrong here, disabled it for now.
-                if False and \
-                    variable_trace.isAssignTrace() and \
-                   not variable_trace.getAssignNode().getAssignSource().\
-                     mayHaveSideEffects() and \
-                   not variable_trace.getPotentialUsages():
-                    variable_trace.getAssignNode().replaceWith(None)
-
 
     def makeVariableTraceOptimizations(self, owner):
         # Reliable trace based optimization goes here:
@@ -144,7 +136,7 @@ class CollectionTracingMixin:
             self.addVariableTrace(
                 variable = variable,
                 version  = version,
-                trace    = VariableUnknownTrace(
+                trace    = VariableTraceUnknown(
                     variable = variable,
                     version  = version,
                     previous = current
@@ -187,7 +179,7 @@ class CollectionStartpointMixin:
         return result
 
     def getVariableTracesAll(self):
-        return self.variable_traces.values()
+        return self.variable_traces
 
     def addVariableTrace(self, variable, version, trace):
         key = variable, version
@@ -197,7 +189,7 @@ class CollectionStartpointMixin:
 
     def addVariableMergeTrace(self, variable, trace_yes, trace_no):
         version = variable.allocateTargetNumber()
-        trace_merge = VariableMergeTrace(
+        trace_merge = VariableTraceMerge(
             variable  = variable,
             version   = version,
             trace_yes = trace_yes,
@@ -206,10 +198,6 @@ class CollectionStartpointMixin:
 
         self.addVariableTrace(variable, version, trace_merge)
 
-        # Merging is using, might imply releasing.
-        trace_yes.addUsage(trace_merge)
-        trace_no.addUsage(trace_merge)
-
         return version
 
     def dumpTraces(self):
@@ -223,7 +211,7 @@ class CollectionStartpointMixin:
         self.addVariableTrace(
             variable = variable,
             version  = 0,
-            trace    = VariableUnknownTrace(
+            trace    = VariableTraceUnknown(
                 variable = variable,
                 version  = 0,
                 previous = None
@@ -236,7 +224,7 @@ class CollectionStartpointMixin:
         self.addVariableTrace(
             variable = variable,
             version  = 0,
-            trace    = VariableInitTrace(
+            trace    = VariableTraceInit(
                 variable = variable,
                 version  = 0,
             )
@@ -248,7 +236,7 @@ class CollectionStartpointMixin:
         self.addVariableTrace(
             variable = variable,
             version  = 0,
-            trace    = VariableUninitTrace(
+            trace    = VariableTraceUninit(
                 variable = variable,
                 version  = 0,
                 previous = None
@@ -263,6 +251,9 @@ class CollectionStartpointMixin:
     def hasUnclearLocals(self):
         return self.unclear_locals
 
+    def updateFromCollection(self, old_collection):
+        VariableRegistry.updateFromCollection(old_collection, self)
+
 
 class ConstraintCollectionBase(CollectionTracingMixin):
     def __init__(self, parent):
@@ -342,7 +333,7 @@ class ConstraintCollectionBase(CollectionTracingMixin):
         version = variable_ref.getVariableVersion()
         variable = variable_ref.getVariable()
 
-        variable_trace = VariableAssignTrace(
+        variable_trace = VariableTraceAssign(
             assign_node = assign_node,
             variable    = variable,
             version     = version,
@@ -373,7 +364,7 @@ class ConstraintCollectionBase(CollectionTracingMixin):
 
         old_trace = self.getVariableCurrentTrace(variable)
 
-        variable_trace = VariableUninitTrace(
+        variable_trace = VariableTraceUninit(
             variable = variable,
             version  = version,
             previous = old_trace
@@ -567,9 +558,6 @@ class ConstraintCollectionFunction(CollectionStartpointMixin,
                                    ConstraintCollectionBase,
                                    VariableUsageTrackingMixin):
     def __init__(self, parent, function_body):
-        # Too complex stuff here, but could be improved, just not now.
-        # pylint: disable=R0912
-
         assert function_body.isExpressionFunctionBody(), function_body
 
         CollectionStartpointMixin.__init__(self)
@@ -580,6 +568,9 @@ class ConstraintCollectionFunction(CollectionStartpointMixin,
         )
 
         self.function_body = function_body
+
+        # TODO: Move this to computeFunction method of functions.
+        old_collection = function_body.constraint_collection
         function_body.constraint_collection = self
 
         statements_sequence = function_body.getBody()
@@ -597,66 +588,14 @@ class ConstraintCollectionFunction(CollectionStartpointMixin,
             if result is not statements_sequence:
                 function_body.setBody(result)
 
+        function_body.constraint_collection.updateFromCollection(old_collection)
+
         self.makeVariableTraceOptimizations(function_body)
 
-        if not Options.isExperimental() or self.removes_knowledge:
-            return
-
-        # self.dumpTrace()
-
-        # Cannot mess with locals yet.
-        if self.unclear_locals:
-            return
-
-        # TODO: Merge this into makeVariableTraceOptimizations instead, could
-        # check the above for these too.
-        # Trace based optimization goes here:
-        for variable_trace in self.variable_traces.values():
-            variable = variable_trace.getVariable()
-
-            # print variable
-
-            if variable.isLocalVariable() and not isSharedLogically(variable):
-                if variable_trace.isAssignTrace():
-                    assign_node = variable_trace.getAssignNode()
-
-                    if not assign_node.getAssignSource().mayHaveSideEffects():
-
-                        if not variable_trace.getPotentialUsages() and \
-                           not variable_trace.isEscaped():
-                            assign_node.parent.replaceWith(
-                                StatementDelVariable(
-                                    variable_ref = assign_node,
-                                    tolerant     = True,
-                                    source_ref   = assign_node.getSourceReference()
-                                )
-                            )
-
-                            for release in variable_trace.releases:
-                                if release.isStatementDelVariable():
-                                    release.replaceWith(
-                                        None
-                                    )
-
-                            self.signalChange(
-                                "new_statements",
-                                assign_node.parent.getSourceReference(),
-                                "Removed assignment without effect."
-                            )
-                elif variable_trace.isMergeTrace():
-                    # print variable_trace
-                    if not variable_trace.getDefiniteUsages() and \
-                       not variable_trace.isEscaped() and \
-                       not variable_trace.releases:
-                        pass
-                        # print "HIT", variable_trace
-
-
     def __repr__(self):
         return "<ConstraintCollectionFunction for %s>" % self.function_body
 
 
-
 class ConstraintCollectionModule(CollectionStartpointMixin,
                                  ConstraintCollectionBase,
                                  VariableUsageTrackingMixin):
