@@ -38,7 +38,10 @@ from nuitka.nodes.StatementNodes import (
     StatementsFrame,
     StatementsSequence
 )
-from nuitka.nodes.TryFinallyNodes import ExpressionTryFinally, StatementTryFinally
+from nuitka.nodes.TryFinallyNodes import (
+    ExpressionTryFinally,
+    StatementTryFinally
+)
 
 
 def dump(node):
