@@ -51,6 +51,7 @@ PACKAGE_LIST = (
     "nuitka",
     "nuitka/nodes",
     "nuitka/tree",
+    "nuitka/importing",
     "nuitka/build",
     "nuitka/freezer",
     "nuitka/gui",
@@ -59,6 +60,8 @@ PACKAGE_LIST = (
     "nuitka/optimizations",
     "nuitka/finalizations",
     "nuitka/plugins",
+    "nuitka/containers",
+    "nuitka/utils",
 )
 
 def diffRecursive(dir1, dir2):
